import os

from bottle import Bottle, TEMPLATE_PATH, debug
from bottle import response

from .routes import register_routes
from .settings import APP_ROOT

TEMPLATE_PATH.insert(0, os.path.join(APP_ROOT, 'dist'))


def create_app(adr_path: str) -> Bottle:
    app = Bottle()
    app.config.load_dict({
        'adr_path': adr_path
    })
    register_routes(app)

    @app.hook('after_request')
    def enable_cors() -> None:
        response.headers['Access-Control-Allow-Origin'] = '*'
        response.headers['Access-Control-Allow-Methods'] = \
            'PUT, GET, POST, DELETE, OPTIONS'
        response.headers['Access-Control-Allow-Headers'] = \
            'Origin, Accept, Content-Type, X-Requested-With, X-CSRF-Token'

    debug(True)
    return app



import os

import click

from adre.utils import (
    filename_extract_number,
    create_adr,
    list_adr_files
)
from adre.server.app import create_app


class ContextObject:
    def __init__(self, path: str):
        self.path = path


@click.group()
@click.pass_context
@click.option(
    '-p', '--path', default='doc/adr/', help='where your adr\'s are stored')
def main(ctx: click.Context, path: str) -> None:
    ctx.obj = ContextObject(
        path=path
    )


@click.command()
@click.pass_obj
def init(ctx_obj: ContextObject) -> None:
    path = ctx_obj.path
    os.makedirs(path, exist_ok=True)
    click.echo('ADR folder initialized in {path}'.format(path=path))


@click.command()
@click.pass_obj
@click.argument('title')
def new(ctx_obj: ContextObject, title: str) -> None:
    """create a new adr with the given title
    """
    path = ctx_obj.path
    filenames = list_adr_files(path)
    adr_numbers = list(map(
        lambda fname: filename_extract_number(fname),
        filenames
    ))
    adr_numbers = sorted(adr_numbers, reverse=True)
    next_number = 1
    if len(adr_numbers):
        next_number = adr_numbers[0] + 1
    adr_path = create_adr(title, next_number, path, 'Proposed')
    click.echo('ADR created: {adr_path}'.format(adr_path=adr_path))


@click.command()
@click.pass_obj
def serve(ctx_obj: ContextObject) -> None:
    web_app = create_app(ctx_obj.path)
    web_app.run(reloader=True)


main.add_command(init)
main.add_command(new)
main.add_command(serve)

if __name__ == "__main__":
    main()



from typing import Dict

from bottle import view, static_file, Response, request

from adre.utils import extract_adrs
from .settings import STATIC_ROOT


@view('index')
def home() -> Dict:
    return {}


def adrs_list() -> Dict:
    adrs = extract_adrs(request.app.config.adr_path)
    return {
        'adrs': [
            {
                'id': adr.id,
                'title': adr.title,
                'date': adr.date.isoformat(),
                'tags': adr.tags,
                'content': adr.content,
            } for adr in adrs
        ]
    }


def static(filepath: str) -> Response:
    return static_file(filepath, STATIC_ROOT)



import re
import os
import datetime
from typing import List

from .models import ADR

ROOT = os.path.dirname(os.path.abspath(__file__))


def list_adr_files(path: str) -> List[str]:
    filenames = os.listdir(path)
    adrs = list(filter(
        lambda fname: filename_extract_number(fname),
        filenames
    ))
    return adrs


def filename_extract_number(filename: str) -> int:
    return ADR(filename=filename, content='').id


def create_adr(
        title: str, number: int, path: str, status: str = 'Proposed'
) -> str:
    template_path = os.path.join(ROOT, 'adr_template.md')
    title_slug = re.sub(r'[\W_]', '-', title)
    adr_path = os.path.join(
        path,
        '{number:04d}-{title_slug}.md'.format(
            number=number,
            title_slug=title_slug
        )
    )
    template_string = ''
    with open(template_path, 'r') as template_file:
        template_string = template_file.read()

    with open(adr_path, 'w') as output_file:
        output_file.write(template_string.format(
            number=number,
            title=title,
            date=datetime.date.today().isoformat(),
            status=status
        ))
    return adr_path


def extract_adrs(path: str) -> List[ADR]:
    files = list_adr_files(path)
    adrs = []
    for file in files:
        with open(os.path.join(path, file), 'r') as f:
            adrs.append(ADR(file, f.read()))
    return adrs



import datetime
import re
from typing import List

FILENAME_NUMBER_REGEX = re.compile(r'^\d{4}')
ADR_TITLE_REGEX = re.compile(r'^#\s\d+\.\s(.+)', re.MULTILINE)
ADR_DATE_REGEX = re.compile(r'^date:\s*(.+)', re.MULTILINE | re.IGNORECASE)
ADR_TAG_SECTION_REGEX = re.compile(
    r'#+\s*tags([^#]*)',
    re.DOTALL | re.IGNORECASE
)
ADR_TAG_REGEX = re.compile(r'^\s*([-+*])\s*(.+)', re.MULTILINE | re.IGNORECASE)


class ADR:
    def __init__(self, filename: str, content: str):
        self.filename = filename
        self.content = content

    @property
    def id(self) -> int:
        match = FILENAME_NUMBER_REGEX.match(self.filename)
        if match:
            return int(match.group(0))
        raise Exception(
            '{filename} doesn\'t appear to be a valid ADR filename'.format(
                filename=self.filename))

    @property
    def title(self) -> str:
        match = ADR_TITLE_REGEX.search(self.content)
        if match:
            return match.group(1)
        raise Exception(
            'Could not find title for adr.'
        )

    @property
    def date(self) -> datetime.date:
        match = ADR_DATE_REGEX.search(self.content)
        if match is None:
            raise Exception(
                'Could not find date for adr.'
            )
        return datetime.datetime.strptime(
            match.group(1),
            '%Y-%m-%d'
        ).date()

    @property
    def tags(self) -> List[str]:
        match = ADR_TAG_SECTION_REGEX.search(self.content)
        if match is None:
            return []
        content = match.group(1)
        tags = ADR_TAG_REGEX.findall(content)
        return [tag[1] for tag in tags]



from bottle import Bottle

from . import controllers


def register_routes(app: Bottle) -> None:
    app.route('/', ['GET'], controllers.home)
    app.route('/api/adrs', ['GET'], controllers.adrs_list)
    app.route('/static/<filepath:path>', ['GET'], controllers.static)



from setuptools import setup, find_packages

with open('README.md', 'r') as fh:
    long_description = fh.read()

setup(
    name='adre',
    version='0.0.1',
    description='ADR Extended',
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=find_packages(),
    install_requires=[
        'click',
        'bottle'
    ],
    entry_points={
        'console_scripts': [
            'adre=adre.cli:main'
        ]
    },
    include_package_data=True,
)
